/**
 * 手机模型
 * 可以切换背景颜色、填充方式
 */
import React, {
  useState,
  useRef,
  useEffect,
  useCallback,
  useImperativeHandle,
} from 'react';
import SVGA from 'svgaplayerweb';
import styles from './Phone.less';
import { blobToDataURL } from '@/utils/base';
import checkBlack from '@/assets/icon_check.png'
import checkWhite from '@/assets/icon_check_white.png'

type bgFitType =
  | 'contain'
  | 'cover'
  | 'fill'
  | '-moz-initial'
  | 'inherit'
  | 'initial'
  | 'revert'
  | 'unset'
  | 'none'
  | 'scale-down'
  | undefined;

// 填充方式
const fillArray: Array<{ key: string; bgFit: bgFitType, name: string }> = [
  { key: 'AspectFit', bgFit: 'contain', name: '原始尺寸' },
  { key: 'AspectFill', bgFit: 'cover', name: '铺满' },
  { key: 'Fill', bgFit: 'fill', name: '拉伸' },
];

// 背景颜色
const bgColors = [
  { color: '#ffffff' },
  { color: '#000000' },
  { color: '#FF5353' },
  { color: '#FF9840' },
  { color: '#FFE62B' },
  { color: '#63EF74' },
  { color: '#4DADFF' },
  { color: '#C15DFF' },
];

const Phone = ({ currentPhone, bgImage }: any, ref?: any) => {
  // 填充方式
  const [fillType, setFillType] = useState(fillArray[0].key);
  const [bgFit, setBgFit] = useState(fillArray[0].bgFit);
  const [player, setPlayer]: any = useState();
  const [parser, setParser]: any = useState();
  const [bgColor, setBgColor] = useState(bgColors[0].color);
  const [videoItem, setVideoItem] = useState();

  useEffect(() => {
    let _player: any = new SVGA.Player('#demoCanvas');
    let _parser: any = new SVGA.Parser();
    setPlayer(_player);
    setParser(_parser);
  }, []);

  const fillTypeChange = useCallback(
    item => (e: any) => {
      e.preventDefault();
      setFillType(item.key);
      setBgFit(item.bgFit);
      player.setContentMode(item.key);
    },
    [fillType, player],
  );

  const onFileDrop = (e: any) => {
    e.stopPropagation();
    e.preventDefault();

    const file = e.dataTransfer.files[0];
    if (!file) {
      return;
    }

    let blob = file;

    blobToDataURL(blob, (data: any) => {
      parser.load(data, (_videoItem: any) => {
        _videoItem.nativeFile = file;
        setVideoItem(_videoItem);
        player.setVideoItem(_videoItem);
        player.startAnimation();
      });
    });
  };

  // 颜色切换
  const bgColorChange = useCallback(
    item => (e: any) => {
      e.preventDefault();
      setBgColor(item.color);
    },
    [bgColor],
  );

  useImperativeHandle(ref, () => ({
    videoItem,
  }));

  return (
    <div className={styles['content-wrap']}>
      <div className={styles['fill-type-wrap']}>
        {fillArray.map(item => (
          <a
            href="#"
            key={item.key}
            onClick={fillTypeChange(item)}
            className={fillType === item.key ? styles['active'] : ''}
          >
            {item.name}
          </a>
        ))}
      </div>
      <div 
        className={styles['phone-stage']}
      >
        <div 
          className={styles['phone-wrap']} 
          style={{ 
            transform: 'scale(.68)'
          }}
        >
          <div
            className={styles['phone']}
            style={{
              width: currentPhone.width,
              height: currentPhone.height,
              background: bgColor,
            }}
          >
            {bgImage && (
              <div className={styles['phone-bg-image']}>
                <img src={bgImage} alt="" style={{ objectFit: bgFit }} />
              </div>
            )}

            {!videoItem && (
              <div className={styles['empty']}>
                <img src="/empty.png" alt="" />
                <h3>请将文件拖拽此区域</h3>
              </div>
            )}

            <div
              id="demoCanvas"
              className={styles['phone-player']}
              onDrop={onFileDrop}
              onDragOver={(e: any) => {
                e.preventDefault();
              }}
            />
          </div>
        </div>
      </div>

      <div className={styles['colors-wrap']}>
        {bgColors.map((item, i) => (
          <a
            style={{ background: item.color }}
            key={item.color}
            onClick={bgColorChange(item)}
            className={bgColor === item.color ? styles['active'] : ''}
          >
            <img src={ i === 0 ? checkBlack : checkWhite } alt=""/>
          </a>
        ))}
      </div>
    </div>
  );
};
export default React.forwardRef(Phone);
